;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: cl-user -*-
;;;; ***********************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          bosco.asd
;;;; Version:       0.7
;;;; Project:       bosco - Cocoa application template for OpenMCL
;;;; Purpose:       asdf system definition
;;;; Programmer:    mikel evins
;;;; Date Started:  2006/11/22
;;;;
;;;; $Id: bosco.lisp,v 1.3 2006/12/14 00:12:36 mikel Exp $
;;;; ***********************************************************************
;;;;
;;;; About:
;;;;
;;;; An application template for building full-fledged OSX
;;;; applications in OpenMCL. In this version, effort is primarily on
;;;; making it easy to build Cocoa applications.
;;;;
;;;; Intended use:
;;;;
;;;; 1. A new user installs OpenMCL and wants to quickly and easily
;;;;    build a Mac OS X application. The user runs a
;;;;    "make-new-project" shell script and it populates a directory
;;;;    with all the files needed to build a starter
;;;;    application. Running the generated build script from inside
;;;;    the project directory builds the starter project, installing
;;;;    the necessary files in the right place in the app bundle. When
;;;;    the build script is done, the user can launch the app by
;;;;    double-clicking it.
;;;;
;;;; 2. A user running an editing session in emacs or xemacs wants to
;;;;    create a new project. Calling the Lisp function
;;;;    make-new-project creates a new project directory the same way
;;;;    the shell script would.
;;;;
;;;; 3. A user running an editing session in emacs or xemacs wants to
;;;;    rebuild the application whose source code is being edited.
;;;;    Calling a Lisp function like build-project builds the sources
;;;;    and updates the app bundle to incorporate the newly-compiled code.
;;;;
;;;; Directories and Files in a New Project:
;;;;
;;;; ProjectName
;;;;   build.sh                         build script
;;;;   project-name.asd                 system definition
;;;;   src/                             Lisp sources
;;;;   bin/
;;;;     ProjectName.app                application bundle
;;;;   resources/                       
;;;;     nibs/                          project-specific nibfiles
;;;;                                    (populated at project-creation
;;;;                                     time from templates)
;;;;
;;;; The bosco subsystem contains template resources (app bundle, nibfiles)
;;;; that the project-creation script uses to populate the project.
;;;;
;;;; In order to satisfy requirements (1) and (2), the code that
;;;; actually constructs a new project is lisp code; the shell script
;;;; is simply a driver that invokes openmcl with the expression
;;;; needed to start the project-creation.

(load "templates/src/lib/utils.lisp")

;;; ----------------------------------------------------------------------
;;; project construction utilities
;;; ----------------------------------------------------------------------

(defun make-project-directory (project-path)
  (if (probe-file project-path)
      (error "'~A' already exists" project-path)
      (let* ((src-dir (path project-path (ensure-directory-pathname "src")))
             (lib-dir (path src-dir (ensure-directory-pathname "lib")))
             (bin-dir (path project-path (ensure-directory-pathname "bin")))
             (rsrc-dir (path project-path (ensure-directory-pathname "resources")))
             (nibs-dir (path rsrc-dir (ensure-directory-pathname "nibs"))))
        (ensure-directories-exist lib-dir)
        (ensure-directories-exist bin-dir)
        (ensure-directories-exist rsrc-dir)
        (ensure-directories-exist nibs-dir))))

(defun populate-project-directory (name project-path)
  (let* (
         (template-nibs (directory (path +bosco-home+ "resources" "nibs" "*.nib")
                              :directories t :files nil))
         (template-strings (path +bosco-home+ "resources" "nibs" "InfoPlist.strings"))
         (template-credits (path +bosco-home+ "resources" "nibs" "Credits.rtf"))
         (project-nibdir (path project-path "resources" "nibs"))
         (project-nibs 
          (mapcar #'(lambda (f) (path project-nibdir (first (last (pathname-directory f)))))
                            template-nibs))
         (project-strings (path project-path "resources" "nibs" "InfoPlist.strings"))
         (project-credits (path project-path "resources" "nibs" "Credits.rtf"))
         (template-src-dir (ensure-directory-pathname (path +bosco-home+ "templates" "src")))
         (template-src-files (directory (path template-src-dir "*.lisp")))
         (template-lib-dir (ensure-directory-pathname 
                            (path +bosco-home+ "templates" "src" "lib")))
         (template-lib-files (directory (path template-lib-dir "*.lisp")))
         (project-src-dir (ensure-directory-pathname (path project-path "src")))
         (project-src-files (mapcar #'(lambda (f) (path project-src-dir (file-namestring f)))
                                    template-src-files))
         (project-lib-dir (ensure-directory-pathname (path project-path "src" "lib")))
         (project-lib-files (mapcar #'(lambda (f) (path project-lib-dir (file-namestring f)))
                                    template-lib-files))
         (template-system-def (path +bosco-home+ "templates" "cocoa-template.asd"))
         (project-system-def (path project-path (concatenate 'string name ".asd")))
         (template-build-file (path +bosco-home+ "templates" "build-template.sh"))
         (project-build-file (path project-path (concatenate 'string "build-" name ".sh")))
         (template-clean-file (path +bosco-home+ "templates" "clean-template.sh"))
         (project-clean-file (path project-path (concatenate 'string "clean-" name ".sh")))
         (template-substitutions `(("<<<project-name>>>" . ,name))))
    ;; copy nibs into resource directory 
    (mapc #'(lambda (temp proj) (if (ccl:directoryp temp)
                                    (recursive-copy-directory temp proj)
                                    (ccl:copy-file temp proj)))
          template-nibs project-nibs)
    ;; copy strings and credits into resource directory
    (ccl::copy-file template-strings project-strings)
    (ccl::copy-file template-credits project-credits)
    ;; instantiate lisp files into src dir
    (mapc #'(lambda (temp proj) (instantiate-template-file temp proj template-substitutions))
          template-src-files project-src-files)
    (mapc #'(lambda (temp proj) (instantiate-template-file temp proj template-substitutions))
          template-lib-files project-lib-files)
    ;; instantiate system definition into src dir
    (instantiate-template-file template-system-def project-system-def template-substitutions)
    ;;; instantiate build template into project directory
    (instantiate-template-file template-build-file project-build-file template-substitutions)
    (instantiate-template-file template-clean-file project-clean-file template-substitutions)))

(defun make-application-bundle (name project-path)
  (let* ((app-bundle (path project-path 
                           (ensure-directory-pathname "bin") 
                           (ensure-directory-pathname (concatenate 'string name ".app"))))
         (contents-dir (path app-bundle (ensure-directory-pathname "Contents")))
         (macos-dir (path contents-dir (ensure-directory-pathname "MacOS")))
         (rsrc-dir (path contents-dir  "Resources" (ensure-directory-pathname "English.lproj")))
         (template-appfiles (list (path +bosco-home+ "resources" "app" "Info.plist")
                                  (path +bosco-home+ "resources" "app" "PkgInfo")))
         (project-appfiles (list (path project-path "bin" (concatenate 'string name ".app")
                                       "Contents" "Info.plist")
                                 (path project-path "bin" (concatenate 'string name ".app")
                                       "Contents" "PkgInfo")))
         (template-substitutions `(("<<<project-name>>>" . ,name))))
    (ensure-directories-exist macos-dir)
    (ensure-directories-exist rsrc-dir)
    ;; copy appfiles into app bundle
    (mapc #'(lambda (temp proj) (instantiate-template-file temp proj template-substitutions))
          template-appfiles project-appfiles)
    name))

;;; ----------------------------------------------------------------------
;;; main entry point for project creation
;;; ----------------------------------------------------------------------

(defun make-new-project (&key
                         (name "BoscoProject")
                         (projects-directory (or (ccl::getenv "BOSCO_PROJECTS_HOME")
                                                 ".")))
  (let ((project-path (if (string= projects-directory ".")
                          (merge-pathnames (ensure-directory-pathname name) 
                                           (ccl::current-directory))
                          (merge-pathnames (ensure-directory-pathname name) 
                                           (ensure-directory-pathname projects-directory)))))
    ;; if the project-directory is a relative path, expand it to an absolute
    ;; path by assuming it's relative to the current working directory
    (let ((project-path (or (and (eq (first (pathname-directory project-path)) :absolute)
                                 project-path)
                            (merge-pathnames (ensure-directory-pathname project-path) 
                                              (ccl::current-directory)))))

      ;; 1. make the project directory
      (make-project-directory project-path)
      ;; 2. copy project files and instantiate template files into the
      ;;    new project
      (populate-project-directory name project-path)
      ;; 3. build the app bundle
      (make-application-bundle name project-path)
      project-path)))

